package com.netease.nim.uikit.common.activity

import android.Manifest
import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.widget.Toast
import com.bullet.chat.grpc.ErrorCode
import com.google.protobuf.Empty
import com.netease.nim.avchatkit.activity.AVChatActivity
import com.netease.nim.avchatkit.constant.AVChatExtras
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.api.model.main.LoginSyncDataStatusObserver
import com.netease.nim.uikit.business.preference.UserPreferences
import com.netease.nim.uikit.business.session.audio.MessageAudioControl
import com.netease.nim.uikit.common.ui.dialog.CustomDialogBuilder
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import com.netease.nim.uikit.common.ui.toast.NimToast
import com.netease.nim.uikit.common.util.sys.SysInfoUtil
import com.netease.nim.uikit.common.util.sys.UIHandler
import com.netease.nim.uikit.impl.cache.FriendDataCache
import com.netease.nim.uikit.support.permission.AutoBootPermissionUtil
import com.netease.nim.uikit.support.permission.MPermission
import com.netease.nim.uikit.support.permission.annotation.OnMPermissionDenied
import com.netease.nim.uikit.support.permission.annotation.OnMPermissionGranted
import com.netease.nim.uikit.support.permission.annotation.OnMPermissionNeverAskAgain
import com.netease.nimlib.sdk.NIMClient
import com.netease.nimlib.sdk.NIMSDK
import com.netease.nimlib.sdk.NimIntent
import com.netease.nimlib.sdk.RequestCallback
import com.netease.nimlib.sdk.friend.FriendService
import com.netease.nimlib.sdk.friend.constant.VerifyType
import com.netease.nimlib.sdk.friend.model.AddFriendData
import com.netease.nimlib.sdk.msg.MsgService
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import smartisan.cloud.im.GrpcCallback
import smartisan.cloud.im.GrpcLoginManager
import smartisan.cloud.im.framwork.NimSingleThreadExecutor
import smartisan.cloud.im.log.LogUtil

/**
 * @author hedongjin <a href="mailto:hedongjin@smartisan.com">Contact me.</a>
 * @version 1.0
 * @since 18-7-31
 */
abstract class MainActivityBase : UI() {

    private val TAG = this.javaClass.simpleName

    override fun onCreate(savedInstanceState: Bundle?) {
        setSlideable(false)
        super.onCreate(savedInstanceState)
    }

    protected fun handleBaseLogic() {
        requestPermission()
        observeCacheSync()
        observeLoginSync()
        checkUnactivedFriend()
        checkAutoStartDialog()

        //初始化语音播放器的听筒模式
        MessageAudioControl.getInstance(this).setEarPhoneModeEnable(UserPreferences.isEarPhoneModeEnable())
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        intent?.let { parseIntent(it) }
    }

    private fun requestPermission() {
        MPermission.printMPermissionResult(true, this, BASIC_PERMISSIONS)
        MPermission.with(this)
                .setRequestCode(BASIC_PERMISSIONS_REQUEST_CODE)
                .permissions(*BASIC_PERMISSIONS)
                .request()
    }

    private fun observeLoginSync() {
        if (!LoginSyncDataStatusObserver.getInstance().observeSyncDataCompletedEvent {
            notifyLoginSyncCompleted()
            DialogMaker.dismissProgressDialog()
        }) {
            DialogMaker.showProgressDialog(this, getString(R.string.prepare_data)).setCanceledOnTouchOutside(false)
        } else {
            notifyLoginSyncCompleted()
        }
    }

    open fun observeCacheSync() {
        if (!NimUIKit.isInitComplete()) {
            val runnable = Runnable {
                NimUIKit.setInitStateListener(null)
                notifyCacheSyncCompleted()
            }

            NimUIKit.setInitStateListener {
                UIHandler.removeCallbacks(runnable)
                UIHandler.post(runnable)
            }

            UIHandler.postDelayed(runnable, 3000)
        } else {
            notifyCacheSyncCompleted()
        }
    }

    open fun notifyLoginSyncCompleted() {}

    open fun notifyCacheSyncCompleted() {
        parseIntent(intent)
    }

    protected fun parseIntent(intent: Intent) {
        when {
            TextUtils.isEmpty(NimUIKit.getAccount()) -> handleLogin()
            NIMSDK.getMixPushService().isFCMIntent(intent) -> handleFCMIntent(intent)
            intent.hasExtra(NimIntent.EXTRA_NOTIFY_CONTENT) -> handleNotifyIntent(intent)
            intent.hasExtra(AVChatExtras.EXTRA_FROM_NOTIFICATION) -> handleAVChatNotifyIntent(intent)
            intent.hasExtra(AVChatActivity.INTENT_ACTION_AVCHAT) -> handleAVChatActionIntent(intent)
            intent.hasExtra(EXTRA_APP_QUIT) -> handleQuit(intent)
            intent.hasExtra(EXTRA_APP_KO) -> handleKo(intent)
        }
    }

    abstract fun handleLogin()

    abstract fun handleFCMIntent(intent: Intent)

    abstract fun handleNotifyIntent(intent: Intent)

    abstract fun handleAVChatNotifyIntent(intent: Intent)

    abstract fun handleAVChatActionIntent(intent: Intent)

    abstract fun handleKo(intent: Intent)

    // 注销
    open fun handleQuit(intent: Intent) {
        DialogMaker.showProgressDialog(this, "")
        GrpcLoginManager.getInstance().logout(object : GrpcCallback<Empty>() {
            override fun onSuccess(empty: Empty) {
                LogUtil.grpc("onLogout onSuccess----")
                DialogMaker.dismissProgressDialog()
                handleKo(intent)
            }

            override fun onFailed(code: Int, errorMsg: String) {
                DialogMaker.dismissProgressDialog()
                LogUtil.grpc("onLogout onFailed----:$errorMsg, code:$code")
                if (code == ErrorCode.ERROR_UNAUTHENTICATED_VALUE) {
                    handleKo(intent)
                } else {
                    NimToast.makeToast(applicationContext, errorMsg, Toast.LENGTH_SHORT).show()
                }
            }
        })
    }


    /**
     * 登录时把recentcontact里不是好友的用户加为好友
     * 向非子弹短信用户发送子弹短信，邀请注册方登录时需建立好友关系
     */
    private fun checkUnactivedFriend() {
        LogUtil.d(TAG, "checkUnactivedFriend")
        NimSingleThreadExecutor.getInstance().execute {
            val recentContacts = NIMClient.getService(MsgService::class.java).queryRecentContactsBlock() ?: return@execute

            recentContacts.filter {
                it.sessionType == SessionTypeEnum.P2P
                        && !FriendDataCache.getInstance().isMyFriend(it.contactId)
                        && it.unreadCount > 0
            }.forEach {
                LogUtil.d(TAG, "not friend & unread recentcontact accountid=" + it.contactId + ";name=" + it.fromNick)

                NIMClient.getService(FriendService::class.java).addFriend(AddFriendData(it.contactId, VerifyType.DIRECT_ADD, null))
                        .setCallback(object : RequestCallback<Void> {
                            override fun onSuccess(param: Void?) {
                                LogUtil.d(TAG, "verify add friend success")
                            }

                            override fun onFailed(code: Int) {
                                LogUtil.d(TAG, "add friend failed code =" + code)
                            }

                            override fun onException(exception: Throwable?) {
                                LogUtil.d(TAG, "add friend onException:" + exception)
                            }
                        })
            }
        }
    }

    private var mAutoStartTipDialog: AlertDialog? = null
    private fun checkAutoStartDialog() {
        if (!UserPreferences.getFirstLogin()) return
        UserPreferences.saveFirstLogin(false)

        if (!SysInfoUtil.isSmartisanDevice()) {
            mAutoStartTipDialog = CustomDialogBuilder.createCustomDialogBuilder(this)
                    .setTitle(R.string.auto_start_dialog_title)
                    .setMessage(R.string.auto_start_dialog_message)
                    .setPositiveButton(R.string.auto_start_dialog_button, View.OnClickListener {
                        AutoBootPermissionUtil.startDeviceAutoBootActivity(this)
                        mAutoStartTipDialog!!.dismiss()
                    })
                    .setCancelable(false)
                    .create()
            mAutoStartTipDialog!!.show()
        } else {
            showTagView()
        }

    }

    open protected fun showTagView() {
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        MPermission.onRequestPermissionsResult(this, requestCode, permissions, grantResults)

        supportFragmentManager.fragments?.forEach {
            it.onRequestPermissionsResult(requestCode, permissions, grantResults)
        }
    }

    @OnMPermissionGranted(BASIC_PERMISSIONS_REQUEST_CODE)
    fun onBasicPermissionSuccess() {
        MPermission.printMPermissionResult(false, this, BASIC_PERMISSIONS)
    }

    @OnMPermissionDenied(BASIC_PERMISSIONS_REQUEST_CODE)
    @OnMPermissionNeverAskAgain(BASIC_PERMISSIONS_REQUEST_CODE)
    fun onBasicPermissionFailed() {
        NimToast.makeToast(this, "未全部授权，部分功能可能无法正常运行！", Toast.LENGTH_SHORT).show()
        MPermission.printMPermissionResult(false, this, BASIC_PERMISSIONS)
    }

    companion object {
        const val EXTRA_APP_QUIT = "APP_QUIT"
        const val EXTRA_APP_KO = "APP_KO"
        const val BASIC_PERMISSIONS_REQUEST_CODE = 100
        val BASIC_PERMISSIONS = arrayOf(
                Manifest.permission.READ_PHONE_STATE,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.READ_EXTERNAL_STORAGE,
                Manifest.permission.CAMERA,
                Manifest.permission.RECORD_AUDIO,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.READ_CALL_LOG)

    }

}